var map;
var chart;

var start_year = 2000;
var slider_steps = 2016 - start_year;

var json_name = 'data/poverty.json';

var graphTitle = 'Δείτε την εξέλιξη του αριθμού σε αυτή τη χώρα στην πορεία του χρόνου.';
var graphKnobsTitle = 'Εκατομμύρια άνθρωποι';
var graphYName = 'Εκατομμύρια άνθρωποι';

$(document).ready(function() {

    var mapData;

    var finalData  = [];

    $.getJSON(json_name, function(data){
        mapData = data;

        for(i = 0; i <= slider_steps; i++){
            var tempData  = [];
            var year = i+start_year;
            mapData.map(function(elem){
                var str_val = elem[year.toString()].toString();
                var con_val = str_val.replace(/,/, '.');
                if(con_val!==''&&elem.id!=='')
                tempData.push({ id: elem.id, value: parseFloat(con_val).toFixed(2), name: elem['Country Name']});
            });
            finalData.push(tempData);
        }
    }).then(function(){
         map = AmCharts.makeChart( "chartdiv", {
            "type": "map",
            "theme": "none",
            "colorSteps": 100,
            "maxValue": 60,
            "backgroundColor": "#FFFFFF",
            "dataProvider": {
                "mapURL": "assets/worldHigh.svg",
                "zoomLevel": 0.9,
                "areas": finalData[0]
            },
            "areasSettings": {
                "autoZoom": true,
                "balloonText": "[[title]]: <strong>[[value]]</strong>",
                "unlistedAreasColor": "#000000",
                "unlistedAreasAlpha": 0.1,
                "selectedColor" : undefined,
                "selectedOutlineColor": "#716aca",
                "colorSolid": "#ff0000",
            },
            "mapObject": {
              "color": "#630000"
            },
            "smallMap": {
          		"enabled": true,
              "bottom": 10,
              "left": 10,
              "right": NaN,
              "size": 0.2,
              "backgroundColor": "#2b2b2bc4",
              "rectangleColor": "#ff0000",
          	},
             "listeners": [
             {
                 "event": "clickMapObject",
                 "method": function(e) {

                     // Ignore any click not on area
                     if (e.mapObject.objectType !== "MapArea")
                         return;

                     // console.log('selected', e.mapObject.id);
                     selectCountry('#'+e.mapObject.id);

                     // this scrolls the countries list but also scrolls the entire page
                     $('#'+e.mapObject.id).get(0).scrollIntoView({ behavior: 'smooth', block: 'nearest', inline: 'start' });
                 },
             },
             {
                 "event": "dataUpdated",
                 "method": function() {
                     reSelectCountry();
                 }
             }
             ],
            //  "titles": [ {
            //     "text": "Ποσοστό γυναικών που διδάσκουν στα Πανεπιστήμια.",
            //     "size": 14
            // }, {
            //     "text": "πηγή: GenderStats",
            //     "size": 11
            // } ],
            "valueLegend": {
                "right": 10,
                "showAsGradient": true,
            },
            "legend": {
                "width": "100%",
                "marginRight": 27,
                "marginLeft": 27,
                "equalWidths": false,
                "backgroundAlpha": 0.5,
                "backgroundColor": "#FFFFFF",
                "borderColor": "#ffffff",
                "borderAlpha": 1,
                "top": 450,
                "left": 0,
                "horizontalGap": 10,
                "generateFromData": true,
            },
            "zoomControl": {
                "minZoomLevel": 0.9
            },
            "numberFormatter": {
                "precision": -1,
                "decimalSeparator": ",",
                "thousandsSeparator": "."
            },
        } );

        //initialize tables
        updateTable(0);
        //set country table on click zoom listeners
        setCountryTableListeners();
    });

    // listener for map
    $('#selected_year').on('change', function(){
        var sel = $(this).val();
        $('.cur_year span').text(sel);
        var ind = sel - start_year;

        //update map 1
        map.dataProvider.areas = finalData[ind];
        map.validateData();
        updateTable(ind);

        var cur_id = $('.cur_id span').text();
        // console.log('must zoom to ', cur_id);
        map.selectObject(map.getObjectById(cur_id));
        map.zoomToSelectedObject(cur_id);
        setTimeout(selectCountry('#'+cur_id), 180);

        updateCountryValue(cur_id);

        // setCountryAndValue(undefined);
    });

    $('.clear-btn').on('click', function(){
       setCountryAndValue(undefined);
       $('#countries tr.selected').removeClass('selected');
    });

    $('.show-single').on('click', function () {
        countryChartPopup($('.cur_id span').text(), $('.cur_country span').text());
    });


    function countryChartPopup(id, country_name) {

        var span = document.createElement("span");
        span.innerHTML = '' +
            '<div class="chart-modal">' +
                '<div class="row">' +
                    '<div class="display-value">' +
                        'Επιλεγμένη Χώρα: <span>'+country_name+'</span>' +
                    '</div>' +
                '</div>' +
                '<div class="row">' + graphTitle + '</div>' +
                '<div id="chartprogressdiv"></div>' +
            '</div>';

        swal({
            content: span,
            className: 'swal-wide',
            showCloseButton: true,
        });

        setTimeout(function(){
            setupChart(id);
        },100)



    }

    function setupChart(country_id){

        // console.log('selected country id for chart is ', country_id);

        var chartData;

        var tempChartData = [];

        $.getJSON(json_name, function(data){
            chartData = data;
            chartData.map(function(elem){
                if(elem['id']===country_id){
                    for(j=1; j<=slider_steps; j++){
                        var year = (j+start_year).toString();
                        var str_val = elem[year].toString();
                        var con_val = str_val.replace(/,/, '.');
                        if(con_val!==''&&elem.id!=='')
                            tempChartData.push({ date: year, value: parseFloat(con_val).toFixed(2), name: elem['Country Name']});
                    }
                }
            });

        }).then(function(){

            // console.log(tempChartData);

            chart = AmCharts.makeChart("chartprogressdiv", {
                "type": "serial",
                "theme": "light",
                "dataDateFormat": "YYYY",
                "period": "YYYY",
                "precision": 2,
                "valueAxes": [{
                    "id": "v2",
                    "title": graphYName,
                    "gridAlpha": 0,
                    "position": "left",
                    "autoGridCount": false
                }],
                "graphs": [{
                    "id": "g1",
                    "valueAxis": "v2",
                    "bullet": "round",
                    "bulletBorderAlpha": 1,
                    "bulletColor": "#FFFFFF",
                    "bulletSize": 5,
                    "hideBulletsCount": 50,
                    "lineThickness": 2,
                    "lineColor": "#20acd4",
                    "type": "line",
                    "title": graphKnobsTitle,
                    "useLineColorForBulletBorder": true,
                    "valueField": "value",
                    "balloonText": "[[title]]<br /><b style='font-size: 130%'>[[value]]</b>"
                }],
                "chartScrollbar": {
                    "graph": "g1",
                    "oppositeAxis": false,
                    "offset": 30,
                    "scrollbarHeight": 50,
                    "backgroundAlpha": 0,
                    "selectedBackgroundAlpha": 0.1,
                    "selectedBackgroundColor": "#888888",
                    "graphFillAlpha": 0,
                    "graphLineAlpha": 0.5,
                    "selectedGraphFillAlpha": 0,
                    "selectedGraphLineAlpha": 1,
                    "autoGridCount": true,
                    "color": "#AAAAAA"
                },
                "chartCursor": {
                    "pan": true,
                    "valueLineEnabled": true,
                    "valueLineBalloonEnabled": true,
                    "cursorAlpha": 0,
                    "valueLineAlpha": 0.2
                },
                "categoryField": "date",
                "categoryAxis": {
                    "parseDates": true,
                    "dashLength": 1,
                    "minorGridEnabled": true
                },
                "legend": {
                    "useGraphSettings": true,
                    "position": "top"
                },
                "balloon": {
                    "borderThickness": 1,
                    "shadowAlpha": 0
                },
                "export": {
                    "enabled": false
                },
                "dataProvider": tempChartData,
                "numberFormatter": {
                    "precision": -1,
                    "decimalSeparator": ",",
                    "thousandsSeparator": "."
                },
            });
        });
    }

    function updateTable(index) {
        $('#countries tbody').html('');
        if(finalData[index]!==undefined) {
            finalData[index].forEach(function (elem) {
                var text1 = '<td class="country_name">' + elem.name + '</td>';
                var text2 = '<td class="country_id">' + elem.id + '</td>';
                var text3 = '<td class="country_value">' + elem.value.toString().replace('.', ',') + '</td>';
                var text4 = '<td class="country_number_value">' + elem.value + '</td>';
                $('#countries tbody').append('<tr id="'+elem.id+'" class="item">' + text1 + text2 + text3 + text4 + '</tr>');
            });
            setCountryTableListeners();
        }
    }

    function selectCountry(selector_id){
        $(selector_id).click();
    }

    function reSelectCountry(){
        var id = $('.cur_id span').text();
        // console.log('the id ' ,id)
        if(id!=='' && id!== null) {
            // console.log('will zoom back to ', id);
            selectCountry('#'+id);
            $('#'+id).get(0).scrollIntoView({ behavior: 'smooth', block: 'nearest', inline: 'start' });

        }
    }

    function setCountryTableListeners(){
        $('#countries tbody tr').on('click', function () {
            $('#countries tr.selected').removeClass('selected');
            $(this).addClass('selected');
            // console.log('listener is set.');
            var id = $(this).find('.country_id').text();
            var name = $(this).find('.country_name').text();
            var value = $(this).find('.country_value').text();
            setCountryAndValue(id, name, value);
            // console.log('selected ' + id);
            map.selectObject(map.getObjectById(id));
            map.zoomToSelectedObject(map.getObjectById(id));
        });
    }

    function setCountryAndValue(id, name, value){
        if(id!==undefined) {
            $('.cur_country span').text(name);
            $('.cur_value span').text(value);
            $('.cur_id span').text(id);
            $('.show-single').removeClass('hidden');
            $('.clear-btn').removeClass('hidden');
        } else {
            $('.cur_country span').text('-');
            $('.cur_value span').text('-');
            $('.cur_id span').text('');

            $('.show-single').addClass('hidden');
            $('.clear-btn').addClass('hidden');
            map.selectObject(map.getObjectById(id));
            map.zoomToSelectedObject(map.getObjectById(id));
        }
    }

    function updateCountryValue(id){
        var value = $('#'+id).find('.country_value').text();
        if(value!=='' && value!= null){
            $('.cur_value span').text(value);
        } else {
            $('.cur_value span').text('-');
        }
    }

});
